#!/bin/pfsh -- # perl, to stop looping
eval 'exec /usr/perl5/bin/perl -w -S $0 ${1+"$@"}'
  if 0;
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)oracle-script.pl	1.18	01/08/23 SMI"
#
# Configure system for Oracle

use lib '/opt/SUNWscvw/lib/perl';
use Sun::Solaris::Utils qw(gettext);
use Cluster::Cgi;
use Cluster::Script;
use strict;
use Cluster::Common;

(new Cluster::Common)->run_script(\&run_script);

sub run_script {
    my ($q, $outfd) = @_;

    if ($< != 0) {
	die(gettext("Not running as root"));
    }

    # param(confhost) is the host on which we perform the configuration.
    # Here we set param(doconf) to true if we are that host, false otherwise.
    my $hostname = `hostname`;
    chop $hostname;
    my ($confhost) = $q->param('confhost');
    $q->param('doconf', $hostname eq $confhost);

    # Generate the s_dbSid parameter by truncating the globalDBName
    my ($dbSid);
    ($dbSid = $q->param('s_globalDBName')) =~ s/\..*//;
    $q->param('s_dbSid', $dbSid);

    #use strict;
    if ($q->param('mode') eq "installpkgs") {
	&addpkgs($q, $outfd);
    } elsif ($q->param('mode') eq "configure") {
	&configure($q, $outfd);
    } elsif ($q->param('mode') eq "install") {
	&install($q, $outfd);
    } elsif ($q->param('mode') eq "etcsystem") {
	&etcsystem($q, $outfd);
    } elsif ($q->param('mode') eq "rootsh") {
	&rootsh($q, $outfd);
    } else {
	die(sprintf(gettext("Bad mode %s"), $q->param('mode')));
    }
}

# Run the Oracle installer
sub install {
    my ($q, $outfd) = @_;
    # Validate input
    my $i;
    foreach $i ('mode', 'confhost', 'ORACLE_BASE', 'ORACLE_HOME',
	    'CDROM', 's_mountPoint', 's_globalDBName', 'DISPLAY', 'gmode') {
	($q->param($i) ne "") || die(sprintf(gettext("Parameter %s undefined"), $i));
    }

    &shm($q, $outfd);
    &groups($q, $outfd);
    &passwd($q, $outfd);
    &nsswitch($q, $outfd);
    &dirs($q, $outfd); # uses ORACLE_HOME, ORACLE_BASE, s_mountPoint
    &libX11($q, $outfd);

    if ($q->param('gmode') eq "local" || $q->param('doconf')) {
	# Run on all nodes for local installation and only confhost for global
	&rsp($q, $outfd); # Uses most parameters
	&run_installer($q, $outfd); # uses CDROM, ORACLE_HOME, ORACLE_BASE, DISPLAY
    }
}

# Do post-installation configuration changes
sub configure {
    my ($q, $outfd) = @_;
    # Validate input
    my $i;
    foreach $i ('mode', 'confhost', 'ORACLE_HOME', 's_globalDBName',
	    'host', 'logfile', 'listener', 'connect', 'gmode', 'tmpdir') {
	print STDERR $i." ".$q->param($i)."\n"; #XXX
	($q->param($i) ne "") || die(sprintf(gettext("Parameter %s undefined"), $i));
    }

    if ($q->param('gmode') eq "local") {
	# Install to each node
	&stopora($q, $outfd); # uses ORACLE_HOME
	&stoplistener($q, $outfd); # uses ORACLE_HOME
	&oratab($q, $outfd); # uses s_dbSid, ORACLE_HOME
	if (! $q->param('doconf')) {
	    # Not the database installation host
	    &initsid($q, $outfd); # uses s_dbSid, ORACLE_HOME, tmpdir
	    &orapw($q, $outfd); # uses s_dbSid, ORACLE_HOME, tmpdir
	}
	&listener($q, $outfd); # uses ORACLE_HOME, listener, host, s_globalDBName,
				# s_dbSid
	&logfile($q, $outfd); # uses logfile
	&updatetnsnames($q, $outfd);
    } else {
	# Install only to confhost
	if ($q->param('doconf')) {
	    &stopora($q, $outfd); # uses ORACLE_HOME
	    &stoplistener($q, $outfd); # uses ORACLE_HOME
	    &listener($q, $outfd); # uses ORACLE_HOME, listener, host, s_globalDBName,
				# s_dbSid
	    &logfile($q, $outfd); # uses logfile
	    &updatetnsnames($q, $outfd);
	} else {
	    &oratab($q, $outfd); # uses s_dbSid, ORACLE_HOME
	    &orainst($q, $outfd); # uses tmpdir
	    &updatetnsnames($q, $outfd);
	}
    }
}

# Make sure /etc/system has the proper shmsys entries
sub shm {
    my ($q, $outfd) = @_;
    my $s = system('grep -l shmsys:shminfo /etc/system > /dev/null');
    if ($s) {
	die(gettext("shmsys/semsys entries need to be in /etc/system"));
    }
    if (-M "/etc/system" < -M "/etc/initpipe") {
	print $outfd "<br>****".gettext("/etc/system is new.  The node must be rebooted after adding shmsys/semsys entries to /etc/system.\n");
    }
}

# Add oracle to /etc/passwd if necessary
sub passwd {
    my ($q, $outfd) = @_;
    my $s = system('grep -l "^oracle:" /etc/passwd > /dev/null');
    if ($s == 0) {
	print $outfd gettext("oracle exists in /etc/passwd\n");
	return;
    }
    # Add oracle to /etc/passwd
    open(IN, "</etc/passwd") ||
	die(sprintf(gettext("Couldn't open %s"), "/etc/passwd"));
    open(OUT, ">/etc/passwd.new") ||
	die(sprintf(gettext("Couldn't create %s"), "/etc/passwd.new"));
    while (<IN>) {
	if (/^oracle:/) {
	} else {
	    print OUT $_;
	}
    }
    print OUT "oracle:x:120:521:Oracle Admin:".$q->param('ORACLE_HOME').":/bin/csh\n";
    close(OUT);
    close(IN);
    system("cp /etc/passwd /etc/passwd.old");
    system("mv /etc/passwd.new /etc/passwd");
    system("/usr/sbin/pwconv");
    print $outfd gettext("Added oracle to /etc/passwd\n");
}

# Add dba, oinstall to /etc/group if necessary
sub groups {
    my ($q, $outfd) = @_;
    my $s = system('grep -l "^dba:" /etc/group > /dev/null');
    if ($s == 0) {
	$s = system('grep -l "^oinstall:" /etc/group > /dev/null');
	if ($s == 0) {
	    print $outfd gettext("dba/oinstall exist in /etc/group\n");
	    return;
	}
    }
    my $dba = 0;
    my $oinstall = 0;
    open(IN, "</etc/group") ||
	die(sprintf(gettext("Couldn't open %s"), "/etc/group"));
    open(OUT, ">/etc/group.new") ||
	die(sprintf(gettext("Couldn't create %s"), "/etc/group.new"));
    while (<IN>) {
	if (/^oinstall:/) {
	    $oinstall++;
	} elsif (/^dba:/) {
	    $dba++;
	}
	print OUT $_;
    }
    if (! $dba) {
	print OUT "dba:*:520:root,oracle\n";
    }
    if (! $oinstall) {
	print OUT "oinstall:*:521:root,oracle\n";
    }
    close(OUT);
    close(IN);
    system("cp /etc/group /etc/group.old");
    system("mv /etc/group.new /etc/group");
    print $outfd gettext("Added oinstall and dba to /etc/group\n");
}

# Modify nsswitch.conf if necessary
sub nsswitch {
    my ($q, $outfd) = @_;
    if (! -r "/etc/nsswitch") {
	return;
    }
    if (! -r "/etc/nsswitch.old") {
	system("cp /etc/nsswitch /etc/nsswitch.old");
    }
    open(IN, "</etc/nsswitch.conf") ||
	die(sprintf(gettext("Couldn't open %s"), "/etc/nsswitch.conf"));
    open(OUT, ">/etc/nsswitch.conf.new") ||
	die(sprintf(gettext("Couldn't create %s"), "/etc/nsswitch.conf.new"));
    my $found = 0;
    while (<IN>) {
	if (/^group:/) {
	    if (/nisplus/) {
		$_ = "group: files [NOTFOUND=return] nisplus\n";
	    } elsif (/nis/) {
		$_ = "group: files [NOTFOUND=return] nis\n";
	    } else {
		$_ = "group: files\n";
	    }
	}
	print OUT $_;
	$found++;
    }
    if (! $found) {
	    print OUT "group: files\n";
    }
    close(OUT);
    close(IN);
    system("cp /etc/nsswitch.conf /etc/nsswitch.conf.old");
    system("mv /etc/nsswitch.conf.new /etc/nsswitch.conf");
    print $outfd gettext("Updated /etc/nsswitch.conf\n");
}

# Make /var/opt/oracle and ORACLE_HOME directories if necessary
sub dirs {
    my ($q, $outfd) = @_;

    # Make /var/opt/oracle directory
    my ($dir) = "/var/opt/oracle";
    my ($uid, $gid);
    if (! -d $dir) {
	system("mkdir -p /var/opt");
	mkdir $dir, 0755 || die(sprintf(gettext("Couldn't create %s"), $dir));
	$uid = (getpwnam("oracle"))[2];
	$gid = (getgrnam("oinstall"))[2];
	chown($uid, $gid, $dir) || die(sprintf(gettext("Couldn't chown %s"), $dir));
    }

    # Make ORACLE_HOME directory
    $dir = $q->param('ORACLE_HOME');
    if (! -d $dir) {
	system("mkdir -p $dir");
    }
    $uid = (getpwnam("oracle"))[2];
    $gid = (getgrnam("oinstall"))[2];
    chown($uid, $gid, $dir) || die(sprintf(gettext("Couldn't chown %s"), $dir));

    $dir = $q->param('ORACLE_BASE');
    if (! -d $dir) {
	system("mkdir -p $dir");
    }
    chown($uid, $gid, $dir) || die(sprintf(gettext("Couldn't chown %s"), $dir));

    # Make db directory
    $dir = $q->param('s_mountPoint');
    if (! -d $dir) {
	system("mkdir -p $dir");
    }
    $uid = (getpwnam("oracle"))[2];
    $gid = (getgrnam("dba"))[2];
    chown($uid, $gid, $dir) || die(sprintf(gettext("Couldn't chown %s"), $dir));

    # Create oratab
    open(ORATAB, ">/var/opt/oracle/oratab") ||
	die(sprintf(gettext("Couldn't create %s"), "/var/opt/oracle/oratab"));
    close(ORATAB);
    chown($uid, $gid, "/var/opt/oracle/oratab") ||
	die(sprintf(gettext("Couldn't chown %s"), "/var/opt/oracle/oratab"));

}

# Make libX11.so.4 link if necessary
sub libX11 {
    my ($q, $outfd) = @_;
    if (! -e "/usr/lib/libX11.so.4") {
	symlink("/usr/openwin/lib/libX11.so.4", "/usr/lib/libX11.so.4");

    }
}

# Run the Oracle installer
sub run_installer {
    my ($q, $outfd) = @_;
    $ENV{"DISPLAY"} = $q->param('DISPLAY');
    $ENV{"ORACLE_HOME"} = $q->param('ORACLE_HOME');
    $ENV{"ORACLE_BASE"} = $q->param('ORACLE_BASE');
    my ($from) = $q->param('CDROM');
    chdir "$from" ||
	die(sprintf(gettext("Couldn't chdir %s"), $from));
    unlink("/tmp/silentInstall.log");
    unlink($q->param('ORACLE_HOME')."/root.sh");

    # The reponse file with the answers filled in
    my($response) = "/var/cluster/spm/svrtypical.rsp";

    my $cmd = "echo \"cd $from ; ./runInstaller -responseFile $response < /dev/null\" | su oracle 2>&1";
    print $outfd sprintf(gettext("Running Oracle installer: runInstaller -responseFile %s"), $response);
    open(RUN, "$cmd|") || die(sprintf(gettext("Couldn't run %s"), $cmd));
    my $skipit = 0;
    while (<RUN>) {
	if (/ORACLE_HOME is/) {
	    #print $outfd ".";
	    next;
	}

	# Omit random java errors
	if (/must have a parent in order to/) {
	    $skipit = 1;
	}
	if ($skipit && (/java\.awt/ || /java\.lang/ || /oracle\.ewt/ || /oracle\.sysman/)) {
	    #print $outfd "Skipping $_\n";
	    next;
	}
	$skipit = 0;
	print $outfd sprintf(gettext("Got: %s"), $_);
	if (/The user is root/) {
	    print $outfd "ERROR: ".gettext("Can't run as root!\n");
	}
	if (/refused by server/ || /Can't connect to X11/) {
	    close(RUN);
	    &killprocs($q, $outfd);
	    die(sprintf(gettext("Can't connect to display %s"),
		$ENV{"DISPLAY"}));
	}
	if (/write error/) {
	    print $outfd "ERROR: ".gettext("Write error: disk full?\n");
	}
	if (/libX11.so.4/) {
	    print $outfd "ERROR: ".gettext("Can't find /usr/lib/libX11.so.4!\n");
	}
	if (/The installation.*was successful, but/) {
	    print $outfd gettext("*** Conditional success\n");
	} elsif (/The installation.*was successful/) {
	    print $outfd gettext("*** Success!\n");
	}
	if (/Alert:/) {
	    print $outfd gettext("*** Alert\n");
	} elsif (/Error/) {
	    print $outfd "ERROR\n";
	}
    }
    my ($status) = close(RUN);
    #print $outfd "End status $status\n";

    &checkdir("/tmp/silentInstall.log", $outfd);
    &checkdir($q->param('ORACLE_BASE')."/oraInventory/logs/silentInstall.log",
	$outfd);
}

# Check install log for errors
sub checkdir {
    my ($d, $outfd) = @_;
    if (-e $d) {
	print $outfd "$d\n";
	open(IN, "<$d") || die(sprintf(gettext("Couldn't open %s"), $d));
	while (<IN>) {
	    print $outfd $_;
	    if (/Unable to setup Inventory/) {
		print $outfd gettext("*** Inventory permissions wrong\n");
	    }
	    if (/The installation.*was successful, but/) {
		print $outfd gettext("*** Success, more or less!\n");
	    } elsif (/The installation.*was successful/) {
		print $outfd gettext("*** Success!\n");
	    }
	    if (/Alert:/) {
		print $outfd gettext("*** Alert\n");
	    } elsif (/Error/) {
		print $outfd gettext("*** Error\n");
	    }
	}
	close(IN);
    }
}

# Generate the response file
sub rsp {
    # Substitute the answers in %REPLY into the response file
    my ($q, $outfd) = @_;
    my (%REPLY) = (ORACLE_HOME => $q->param('ORACLE_HOME'),
	s_globalDBName => $q->param('s_globalDBName'),
	FROM_LOCATION => $q->param('CDROM')."/stage/products.jar",
	s_mountPoint => $q->param('s_mountPoint'),
	s_globalDBName => $q->param('s_globalDBName'),
	s_dbSid => $q->param('s_dbSid'),
        UNIX_GROUP_NAME => "oinstall",
        SHOW_SPLASH_SCREEN => "true",
        SHOW_ROOTSH_CONFIRMATION => "false");

    if ($q->param('doconf') && $q->param('installdb') eq "yes") {
	# Install the database
	$REPLY{'OPTIONAL_CONFIG_TOOLS'} = "{\"dbca\"}";
    } else {
	# Don't install the database
	$REPLY{'OPTIONAL_CONFIG_TOOLS'} = "{}";
    }

    # The template for the Oracle installer response file
    # First try the 8.1.6 location
    my ($in) = $q->param('CDROM')."/stage/Response/svrtypical.rsp";
    if (! (-e $in)) {
	# Next try the 8.1.7 location
	$in = $q->param('CDROM')."/response/svrtypical.rsp";
	if (! (-e $in)) {
	    die(sprintf(gettext("Couldn't find response file %s"), $in));
	}
    }
    # The reponse file with the answers filled in
    my ($out) = "/var/cluster/spm/svrtypical.rsp";
    open(IN, "<$in") ||
	die(sprintf(gettext("Couldn't open %s"), $in));
    open(OUT, ">$out") ||
	die(sprintf(gettext("Couldn't create %s"), $out));
    while (<IN>) {
	if (/OPTIONAL_CONFIG_TOOLS/ && ! /dbca/) {
	    # Note: multiple OPTIONAL_CONFIG_TOOLS lines but
	    # we want only the dcba one, so skip the others
	} elsif (/^([^=\s]+)\s*=\s*(.)/) {
	    my ($name, $quote) = ($1,$2);
	    # $name is the LHS of the equals
	    # $quote is the first character on the RHS of the equals, which
	    # we check to see if it is a quotatation mark or <Value...
	    if ($REPLY{$name}) {
		if ($quote eq "\"" || $quote eq "<") {
		    # Quotes around reply
		    $_ = "$name=\"".$REPLY{$name}."\"\n";
		} else {
		    # No quotes around reply (very important!)
		    $_ = "$name=".$REPLY{$name}."\n";
		}
	    }
	}
	print OUT $_;
    }
    close(IN);
    close(OUT);
    print $outfd sprintf(gettext("Generated response file from %s\n"), $in);
}

# Run the root.sh script
sub rootsh
{
    my ($q, $outfd) = @_;
    if (! -d $q->param('ORACLE_HOME')) {
	die("Bad ORACLE_HOME".$q->param('ORACLE_HOME'));
    }

    my $cmd = $q->param('ORACLE_HOME')."/root.sh";
    my $script = new Cluster::Script();
    $script->{alarm_timeout} = 60; # 1 minutes between "waiting"
    $script->{max_timeouts} = 30; # 30 minutes before timeout

    my (%args) = ();
    $script->run($cmd, "script-rootsh", $q, $outfd);
}

# Kill Oracle processes to clean up
sub killprocs
{
    my ($q, $outfd) = @_;
    open(PS, "/usr/bin/ps -ef |");
    while (<PS>) {
	s/^\s+//;
	my ($uid, $pid, $ppid, $t1, $t2, $t3, $t4, $cmd) = split(/\s+/);
	if ($cmd =~ /runInstaller/ || $cmd =~ /stage.Component/) {
	    print $outfd sprintf(gettext("Killing %s %s"), $pid, $_);
	    kill 'HUP', $pid;
	}
    }
    close(PS);

}

# Copy initsid.ora from temp directory, make link and permissions
sub initsid {
    my ($q, $outfd) = @_;
    my $initfile = "init".$q->param('s_dbSid').".ora";
    my $pdir = $q->param('ORACLE_HOME')."/admin/".$q->param('s_dbSid')."/pfile";
    system("mkdir -p $pdir");
    system("cp ".$q->param('tmpdir')."/$initfile $pdir/$initfile");
    my $uid = (getpwnam("oracle"))[2];
    my $gid = (getgrnam("oinstall"))[2];
    chown($uid, $gid, $pdir);
    chown($uid, $gid, "$pdir/$initfile");
    print $outfd sprintf(gettext("Copied %s\n"), "$pdir/$initfile");
    my $cmd = "ln -s $pdir/$initfile ".$q->param('ORACLE_HOME')."/dbs";
    system("$cmd");
    print $outfd "$cmd\n";
}

# Copy orapw from temp directory
sub orapw {
    my ($q, $outfd) = @_;
    my $orafile = "orapw".$q->param('s_dbSid');
    my $dir = $q->param('ORACLE_HOME')."/dbs";
    system("cp ".$q->param('tmpdir')."/$orafile $dir/$orafile");
    my $uid = (getpwnam("oracle"))[2];
    my $gid = (getgrnam("oinstall"))[2];
    chown($uid, $gid, "$dir/$orafile");
    chmod 0640, "$dir/$orafile";
    if (! -e "$dir/$orafile") {
	print $outfd "ERROR: ".sprintf(gettext("%s does not exist\n"), "$dir/$orafile");
    } else {
	print $outfd sprintf(gettext("Copied %s\n"), "$dir/$orafile");
    }
}

# Copy oraInst.loc from temp directory
sub orainst {
    my ($q, $outfd) = @_;
    my $orafile = "oraInst.loc";
    my $dir = "/var/opt/oracle";
    system("cp ".$q->param('tmpdir')."/$orafile $dir/$orafile");
    my $uid = (getpwnam("oracle"))[2];
    my $gid = (getgrnam("dba"))[2];
    chown($uid, $gid, "$dir/$orafile");
    chmod 0640, "$dir/$orafile";
    if (! -e "$dir/$orafile") {
	print $outfd "ERROR: ".sprintf(gettext("%s does not exist\n"), "$dir/$orafile");
    } else {
	print $outfd sprintf(gettext("Copied %s\n"), "$dir/$orafile");
    }
}

# Create the listener file
sub listener {
    my ($q, $outfd) = @_;
    my $listenerdir = $q->param('ORACLE_HOME')."/network/admin";
    system("mkdir -p $listenerdir");
    my $uid = (getpwnam("oracle"))[2];
    my $gid = (getgrnam("oinstall"))[2];
    chown($uid, $gid, $listenerdir) || die(sprintf(gettext("Couldn't chown %s"), $listenerdir));
    rename "$listenerdir/listener.ora", "$listenerdir/listener.ora.old";
    open(OUT, ">$listenerdir/listener.ora") ||
	die(sprintf(gettext("Couldn't create %s"),
	    "$listenerdir/listener.ora"));

    my $listener = $q->param('listener');
    my $host = $q->param('host');
    my $home = $q->param('ORACLE_HOME');
    my $dbname = $q->param('s_globalDBName');
    my $sid = $q->param('s_dbSid');
    print OUT << "EOF";
$listener =
  (DESCRIPTION_LIST =
    (DESCRIPTION =
      (ADDRESS_LIST =
        (ADDRESS = (PROTOCOL = IPC)(KEY = EXTPROC))
      )
      (ADDRESS_LIST =
        (ADDRESS = (PROTOCOL = TCP)(HOST = $host)(PORT = 1521))
      )
    )
    (DESCRIPTION =
      (PROTOCOL_STACK =
        (PRESENTATION = GIOP)
        (SESSION = RAW)
      )
      (ADDRESS = (PROTOCOL = TCP)(HOST = $host)(PORT = 2481))
    )
  )

SID_LIST_$listener =
  (SID_LIST =
    (SID_DESC =
      (SID_NAME = PLSExtProc)
      (ORACLE_HOME = $home)
      (PROGRAM = extproc)
    )
    (SID_DESC =
      (GLOBAL_DBNAME = $dbname)
      (ORACLE_HOME = $home)
      (SID_NAME = $sid)
    )
  )
EOF
    close(OUT);
    print $outfd sprintf(gettext("Created %s/listener.ora\n"), $listenerdir);
}

# Add sid to oratab
sub oratab {
    my ($q, $outfd) = @_;
    my $found = 0;
    if (open(IN, "< /var/opt/oracle/oratab")) {
	my $pattern = $q->param('s_dbSid').':'.$q->param('ORACLE_HOME');
	while (<IN>) {
	    if (/^$pattern/) {
		$found = 1;
		last;
	    }
	}
	close(IN);
    }
    if (! $found) {
	open(OUT, ">> /var/opt/oracle/oratab") || die(gettext("can't append to /var/opt/oracle/oratab"));
	print OUT join(":", $q->param('s_dbSid'), $q->param('ORACLE_HOME'), "N\n");
	close(OUT);
	print $outfd gettext("Added entry to /var/opt/oracle/oratab\n");
    }
}

# Make the log file directory and the log file
sub logfile {
    my ($q, $outfd) = @_;
    my ($dir, $uid, $gid);
    ($dir = $q->param('logfile')) =~ s#/[^/]*$##; # Directory prefix
    system("mkdir -p $dir");
    $uid = (getpwnam("oracle"))[2];
    $gid = (getgrnam("oinstall"))[2];
    chown($uid, $gid, $dir);
    system("touch ".$q->param('logfile'));
    chown($uid, $gid, $q->param('logfile'));
}

# Stop the database if running
sub stopora {
    my ($q, $outfd) = @_;
    my $s = system('ps -ef | grep ora_pmon | grep -v grep > /dev/null');
    if ($s) {
	# Not found
	return;
    }
    $ENV{"ORACLE_HOME"} = $q->param('ORACLE_HOME');
    my $cmd = "echo \"".$q->param('ORACLE_HOME')."/bin/dbshut\" | su oracle 2>&1";
    if (system($cmd) == 0) {
	print $outfd gettext("Stopped oracle.\n");
    }
}


# Stop the listener through the lsnrctl command if the listener is running
sub stoplistener {
    my ($q, $outfd) = @_;
    my $s = system('ps -ef | grep tnslsnr | grep -v grep > /dev/null');
    if ($s) {
	# Not found
	return;
    }
    $ENV{"ORACLE_HOME"} = $q->param('ORACLE_HOME');
    my $cmd = $q->param('ORACLE_HOME')."/bin/lsnrctl stop";
    if (system($cmd) == 0) {
	print $outfd gettext("Stopped listener.\n");
    }
}

# Add packages
sub addpkgs {
    my ($q, $outfd) = @_;

    # Validate input
    my $i;
    foreach $i ('pkglist', 'pkgcd') {
	print STDERR $i." ".$q->param($i)."\n"; #XXX
	($q->param($i) ne "") || die(sprintf(gettext("Parameter %s undefined"), $i));
    }

    my ($status);
    my ($pkglist) = "";
    foreach $i (split(/,/, $q->param('pkglist'))) {
	$status = system("/usr/bin/pkginfo -q $i");
	if ($status) {
		$pkglist .= " $i";
	}
    }
    if ($pkglist) {
	delete @ENV{'BASEDIR', 'CLIENT_BASEDIR'};
	if ((stat($q->param('pkgcd')))[4] != 0) {
	    die(gettext("Installation CD must be owned by root"));
	}
	my ($cmd) = "/usr/sbin/pkgadd -d ".$q->param('pkgcd')." -a ".
	    "/opt/SUNWscvw/cgi-bin/installation/sds/noask_pkgadd $pkglist";
	print $outfd "$cmd\n";
	$status = `$cmd 2>&1`;
	if ($?) {
	    die($q->sprintfn(gettext("pkgadd of %s failed: %s\n"),
		$pkglist, $status));
	}
    }
}

# Update /etc/system on the node to include the shm information
sub etcsystem {
    my ($q, $outfd) = @_;
    # See if /etc/system already contains shm entries.
    my $s = system('grep -l shmsys:shminfo /etc/system > /dev/null');
    if (! $s) {
	print $outfd gettext("/etc/system already contains shmsys entries\n");
	return;
    }
    open(OUT, ">> /etc/system") || die(gettext("can't append to /etc/system"));
    print OUT << "EOF";
* Start of lines added by SunPlex Manager
set shmsys:shminfo_shmmax=4294967295
set shmsys:shminfo_shmmin=1
set shmsys:shminfo_shmmni=100
set shmsys:shminfo_shmseg=10
set semsys:seminfo_semmni=100
set semsys:seminfo_semmsl=1000
set semsys:seminfo_semmns=3000
set semsys:seminfo_semopm=100
set semsys:seminfo_semvmx=32767
* End   of lines added by SunPlex Manager
EOF
    close(OUT);
    print $outfd gettext("Updated /etc/system\n");
}

# Update tnsnames.ora
sub updatetnsnames {
    my ($q, $outfd) = @_;
    my $file = $q->param('ORACLE_HOME')."/network/admin/tnsnames.ora";

    my $dbname = $q->param('s_globalDBName');
    my $sid = $q->param('s_dbSid');

    # Open tnsnames.ora.new
    if (!open(OUT, ">$file.new")) {
	print $outfd sprintf(gettext("Couldn't create %s"), "$file.new");
	close(IN);
	return;
    }

    my $skip = 0;

    # Filter old tnsnames.ora
    if (open(IN, "<$file")) {
	while (<IN>) {
	    if (/^\s*([-\w\.]+)\s*=/) {
		# Find a top level "name = in the tnsnames.ora"
		my $name = $1;
		if (lc($name) eq lc($sid) || lc($name) eq lc($dbname)) {
		    # Skip entry for dbname
		    $skip = 1;
		} else {
		    # Stop skipping for another entry
		    $skip = 0;
		}
	    }
	    if (! $skip) {
		print OUT $_;
	    }
	}
	close(IN);
	system("mv $file $file.old");
    }

    # Add new data
    my $port = 1521;
    my $lh = $q->param('host');
    print OUT << "EOF";
$dbname =
(DESCRIPTION =
  (ADDRESS_LIST =
    (ADDRESS = (PROTOCOL = TCP)
    (HOST = $lh)
    (PORT = $port))
  )
  (CONNECT_DATA = (SID = $sid))
)
EOF
    close(OUT);
    my $uid = (getpwnam("oracle"))[2];
    my $gid = (getgrnam("oinstall"))[2];
    chown($uid, $gid, "$file.new");
    chmod 0644, "$file.new";
    if (system("mv $file.new $file") != 0) {
	die(sprintf(gettext("Couldn't create %s"), $file));
    } else {
	print $outfd sprintf(gettext("Updated %s\n"), $file);
    }
}
