#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)scinstall.pl	1.15	01/07/31 SMI"
#
# Run scinstall -i to install clustering packages, or run the full scinstall
# to install the cluster framework.

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Install;

# Create a cgi object
my $q = new Cluster::Cgi;

# Get the installation mode
my $mode = $q->param("mode");

# Create a install object
my $install = new Cluster::Install;

# Print the header
print $q->header();

# Remove the BASEDIR environment variables before continuing
delete @ENV{'BASEDIR', 'CLIENT_BASEDIR'};

# Get the scinstall path
my $scinstall_path = $install->get_scinstall_path($q->param("cdrom_path"));

# If the mode is "install_sccheck", run "scinstall -i"
if ($mode eq "install_sccheck") {

	# Set the state file
	my $DONE_SCINSTALL_I = "/var/cluster/spm/done_scinstall_i";
	# Remove old state files
	system("/usr/bin/pfexec /usr/bin/rm -f $DONE_SCINSTALL_I");

	# Fork so that we can exit to the parent immediately
	if (fork()) {
		exit;
	}

	# Create the scinstall command to install the packages
	my $scinstall_cmd = "$scinstall_path -i";

	# Syslog the command
	$install->syslog("Running: $scinstall_cmd");

	# Run the command
	my @output = `/usr/bin/pfexec $scinstall_cmd 2>&1`;
	my $ret = ($? / 256);

	# Syslog the result
	$install->syslog("scinstall returned: $ret");
	if ($ret != 0) {
		$install->syslog("scinstall error: " . 
				 join(" ", @output));
	}

	# Touch the state file
	system("/usr/bin/pfexec /usr/bin/touch $DONE_SCINSTALL_I");
	CORE::exit(0);
}

# Fork so that we can exit to the parent immediately
if (fork()) {
    exit;
}

# Local vars
my ($scinstall_results);

# Create the command to run for scinstall
if ($q->param("nodenumber") == 1) {
    
    $scinstall_results = "$scinstall_path -d " . $q->param("cdrom_path") . 
	    " -ik -C " . $q->param("cluster_name") . " -F"; 
    
} else {
    
    $scinstall_results = "$scinstall_path -d " .  $q->param("cdrom_path") . 
	    " -ik -C " . $q->param("cluster_name") . " -N " .
		    $q->param("firstnode");
    
}

# Create the proper switch names. If we're using wildcat, we have to create
# names of the form sw_<adaptername>.
my ($switch1, $switch2);

if ($q->param("transport1") =~ /wrsm/) {
	$switch1 = "sw_" .$q->param("transport1"); 
} else {
	$switch1 = "switch1";
}

if ($q->param("transport2") =~ /wrsm/) {
	$switch2 = "sw_" .$q->param("transport2"); 
} else {
	$switch2 = "switch2";
}

# Create the rest of the scinstall command
$scinstall_results .= " -A trtype=dlpi,name=" .  $q->param("transport1") . 
    " -A trtype=dlpi,name=" .  $q->param("transport2") . 
    " -B $switch1 -B $switch2 -m endpoint=" .  $q->param("nodename") . ":" . 
    $q->param("transport1") .  ",endpoint=$switch1 -m endpoint=" . 
    $q->param("nodename") . ":" .  $q->param("transport2") . 
    ",endpoint=$switch2";

# Syslog the command
$install->syslog("Running: $scinstall_results");

# Run the command
my $ret = system("/usr/bin/pfexec $scinstall_results");

# Syslog the command
$install->syslog("scinstall returned: $ret");

# Sleep for 60 seconds to allow the nameservice cache daemon (nscd) to
# restart.  If the reboot fails, then try multiple reboots to awake
# the nscd.  This workaround is for bug 4458070.  For tracking
# purposes, this workaround has the bug id 4460575.
$install->syslog("sleeping for 60 seconds to allow nameservice cache " .
		 "daemon to restart...");
sleep 60;

# Goodbye cruel world!
my $cmd = "/usr/bin/pfexec /usr/sbin/reboot";
$ret = system($cmd);
while ( $ret != 0 ) {
	$install->syslog("Reboot failed, trying again in 30 seconds...");
	sleep 30;
	$ret = system($cmd);
}

# Exit properly
CORE::exit(0);
