#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#pragma ident   "@(#)Device.pm 1.35     01/07/27 SMI"
#
# Disk Device class

package Cluster::Device;
use strict;
use POSIX;
use Cluster::Common;
use Cluster::RunCommand;
use Sun::Solaris::Utils qw(gettext);
use vars qw(@ISA $VERSION);
$VERSION = '1.00';
@ISA = qw(Cluster::Common);

# Internationalization
# action=
;# gettext("Online")
;# gettext("Offline")

##############################################################################
#
# Class constructor
#
##############################################################################

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless ($self, $class);
    return $self;
}

##############################################################################
#
# Class Variables
#
##############################################################################

# File names and location
my $PFEXEC 	   = '/usr/bin/pfexec';     # Used in execute_cmd()
my $DEVICEIMAGE    = '/images/device/devicegrps_16.gif';
my $VIEW     = '/cgi-bin/device/device_view.pl';
my $CONFIG   = '/cgi-bin/device/device_config.pl';
my $ADD      = '/cgi-bin/device/device_add.pl';
my $SCCONF   = '/usr/cluster/bin/scconf';
my $SCSTAT   = '/usr/cluster/bin/scstat';
my $SCSWITCH = '/usr/cluster/bin/scswitch';
my $METASTAT = '/usr/sbin/metastat';
my $MOUNT    = '/usr/bin/df -k';

# Instantiate cluster objects
my $runcommand     = new Cluster::RunCommand;

# Set boolean constants
use constant TRUE  => 0;
use constant FALSE => 1;

# command output in memory 
my @SCCONFPOUT; 
my @SCSTATDOUT;
my @DG_LIST;

##############################################################################
#
# Class methods
#
##############################################################################

sub device_jscript() {
	return "
	function online_dg_submit(dg) {
                self.location.href = '$CONFIG\?step=2&func=Online&devicegroup=' + dg + '&primary=' + document.inputs.primary.options[document.inputs.primary.selectedIndex].value
        }";
}

# Print the header, view and menu bar
sub device_header {
	my ($self, $q, $headername, $devicegroup) = @_;

	# Start the action bar table
        $q->start_action_bar_table($headername);

	if (!defined $devicegroup) {
		# Create the view menu links
		my @viewlinks = (gettext("Device Group Status Table"), $VIEW,
				gettext("Device Group Topology"),
				"$VIEW\?view=topological");

		# Print the view menu
		$q->view_menu(\@viewlinks, $VIEW);
	}

	# Print the middle of the action table
        $q->mid_action_bar_table();

	# Create the actions menu links
        my @links = (gettext("Bring Online/Switch Primary..."),
		     "$CONFIG\?step=1&devicegroup=$devicegroup&func=Online",

		     gettext("Take Offline..."),
		     "$CONFIG\?step=1&devicegroup=$devicegroup&func=Offline",
		     );

	my @action_helpMarkers = ("devicepm-online-switch",
				  "devicepm-offline");
	$q->load_action_helpMarkers(\@action_helpMarkers);

	# Print the action bar table
        $q->actions_menu(gettext("Action Menu"), \@links);

        # End the action bar table
        $q->end_action_bar_table();
}

sub get_devicegrouplist
{
	return @DG_LIST;
}

# Get device group name, state, nodelist and primary
sub _get_devicegrouplist {
	my (@dg_list, $i, @parts, $line);
	$i = 0;
	for (my $j = 0; $j <= $#SCCONFPOUT; $j++) {
		$line = $SCCONFPOUT[$j];
		if ($line =~ /Device group name:/) {
			@parts = split /(\S+)/, $line;
			$dg_list[$i]{'name'} = $parts[7];
			$line = $SCCONFPOUT[++$j];
			@parts = split /(\S+)/, $line;
			$dg_list[$i]{'type'} = $parts[7];	
			$line = $SCCONFPOUT[++$j];
			@parts = split /(\S+)/, $line;
			if ($parts[9] eq "yes") {
				$dg_list[$i]{'failback'} = 
					gettext("enabled");  
			} else {
				$dg_list[$i]{'failback'} = 
					gettext("disabled");
			}
			$line = $SCCONFPOUT[++$j];
			@parts = split /(\:)/, $line;
			$parts[2] =~ s/\s+//g;
			$dg_list[$i]{'nodelist'} = $parts[2]; 

			my @output = $runcommand->scstat("-D");
			for (my $k = 0; $k <= $#output; $k++) {
				$line = $output[$k];
				if ($line =~ /Device group servers:/) {
					@parts = split /(\S+)/, $line;
					if (!($parts[7] eq 
					      $dg_list[$i]{'name'})) {
						next;
					}
					$parts[9] =~ s/\s+//g;
					if (!($parts[9] eq "-")) {
						$dg_list[$i]{'primary'} = 
							$parts[9];
						$dg_list[$i]{'status'} = 
							"Online";  
					} else {
						$dg_list[$i]{'status'} = 
							"Offline";
						$dg_list[$i]{'primary'} = 
							gettext("N/A"); 
					}
					last;
				}
			}
			$i++;
		}
	}
	return (@dg_list);
}

# Get all the mounted directories on this diskgroup.
sub get_mounted_directories($$) {
	my ($self, $dg_name) = @_;

	my @dg_list = $self->get_devicegrouplist();
	my ($node, @mounted);
	my $idx = $self->find_dg_by_name(\@dg_list, $dg_name);
	if ($idx < 0) {
		return;
	}
	
	my $cmd = "/usr/sbin/df -k | grep /dev/md/$dg_name";
	my @output = `$PFEXEC $cmd`;
	if ($? != 0) {
		return;
	}
	for (my $i = 0; $i <= $#output; $i++) {
		my @parts = split(/\s+/, $output[$i]);
		$mounted[$i]{'logicaldisk'} = "$parts[0]";  
		$mounted[$i]{'mounted'} = "$parts[5]";  
	}
	return @mounted;
}

sub find_dg_by_name
{
	my ($self, $dg_list_ref, $dg_name) = @_;
	my @dg_list = @$dg_list_ref;
	for (my $i = 0; $i < $#dg_list + 1; $i++) {
		if ($dg_list[$i]{'name'} eq $dg_name) {
			return $i;
		}
	}
	return (-1); # not found
}

sub config_online($$$$$)
{
	my ($self, $q, $step, $devicegroup, $primary) = @_;

	my @dg_list = $self->get_devicegrouplist();
	if ($#dg_list < 0){
		$q->print_title([gettext("Global Devices"), $VIEW,
		    gettext("Bring Online/Switch Primary"), 
			"$CONFIG\?step=1&devicegroup=$devicegroup&func=Online",
		    gettext("Warning")]);
                print "<form>";
                $q->start_message_table("warning");
                print $q->start_p({ class => "message-header-text" });
                print gettext("No Device Group has been Configured on this cluster");
                print $q->end_p();
                $q->end_message_table($q);
                $q->buttons_table(gettext("  Cancel  "), "self.location.href=\"$VIEW\"" );
                print "</form>";
                return;
        }

	if ($step == 1) {
		# Gather infomation
		$q->print_title([gettext("Global Devices"), $VIEW,
		    gettext("Bring Online/Switch Primary")]);
		print "<form name=inputs action=$CONFIG>";
		print $q->start_table({ cellpadding => 5 });
		print $q->start_Tr();
		print $q->start_td();
		print $q->start_div({ class => "action-window-label-text" });
		print gettext("Device Group:");
		print $q->end_div();
		print $q->end_td();
		print $q->start_td();
		my @dg_links;
		my $j = 0;
		my $selectidx = 0;
		for (my $i = 0; $i <= $#dg_list; $i++) {
			if ($dg_list[$i]{'name'} eq $devicegroup) {
				$selectidx = $i;
			}
			$dg_links[$j++] = $dg_list[$i]{'name'};
			$dg_links[$j++] = "$CONFIG\?step=1&func=Online&devicegroup=$dg_list[$i]{'name'}";
		}
		$q->select_menu(\@dg_links, "dg_menu", $dg_links[1]);
		print $q->start_script();
		print "document.inputs.dg_menu.selectedIndex=$selectidx";
		print $q->end_script();
		print $q->end_td();
		print $q->end_Tr();

		print $q->start_Tr();
		print $q->start_td();
		print $q->start_div({ class => "action-window-label-text" });
		print gettext("Node:");
		print $q->end_div();
		print $q->end_td();
		print $q->start_td();
		my @node_list = $self->get_nodelist_fromstr($dg_list[$selectidx]{'nodelist'});
		print $q->popup_menu({ name => "primary", 
				       values => \@node_list });
		print $q->end_td();
		print $q->end_Tr();
		print $q->end_table();

		$q->buttons_table(gettext(" Bring Online/Switch Primary "),
			"online_dg_submit('$dg_list[$selectidx]{'name'}')",
			gettext("  Cancel  "),
			"self.location.href = \"$VIEW\"" );
		print "</form>";
	} else {
		# Generate command
		my $cmd = "$SCSWITCH -z -D $devicegroup -h $primary";
	
		# Executing the command
		$self->execute_cmd($q, $cmd, "Online", $primary,
			[gettext("Global Devices"), $VIEW,
			    gettext("Bring Online/Switch Primary"),
		     "$CONFIG\?step=1&devicegroup=$devicegroup&func=Online"]);
	}
}

sub config_offline($$$$)
{
	my ($self, $q, $step, $devicegroup) = @_;

	my @dg_list = $self->get_devicegrouplist();
	if ($#dg_list < 0){
		$q->print_title([gettext("Global Devices"), $VIEW,
		    gettext("Take Offline"),
			"$CONFIG\?step=1&devicegroup=$devicegroup&func=Offline",
		    gettext("Warning")]);
                print "<form>";
                $q->start_message_table("warning");
                print $q->start_p({ class => "message-header-text" });
                print gettext("No Device Group has been Configured on this cluster");
                print $q->end_p();
                $q->end_message_table($q);
                $q->buttons_table(gettext("  Cancel  "), "self.location.href=\"$VIEW\"" );
                print "</form>";
                return;
        }

	if ($step == 1) {
		# Gather infomation
		$q->print_title([gettext("Global Devices"), $VIEW,
		    gettext("Take Offline")]);
		print "<form name=inputs action=$CONFIG>";
		print $q->input({ type => "hidden",
                                name => "func",
                                value => "Offline" });
		print $q->input({ type => "hidden",
                                name => "step",
                                value => 2 });
		print $q->start_table({ cellpadding => 5 });
		print $q->start_Tr();
		print $q->start_td();
		print $q->start_div({ class => "action-window-label-text" });
		print gettext("Device Group:");
		print $q->end_div();
		print $q->end_td();
		print $q->start_td();
		print "<select name=devicegroup>";
		my $selectidx = 0;
		for (my $i = 0; $i <= $#dg_list; $i++) {
			if ($dg_list[$i]{'name'} eq $devicegroup) {
				$selectidx = $i;
			}
			print "<option value=\"$dg_list[$i]{'name'}\">$dg_list[$i]{'name'}";
		}
		print "</select>";
		print $q->start_script();
		print "document.inputs.devicegroup.selectedIndex=$selectidx";
		print $q->end_script();
		print $q->end_td();
		print $q->end_Tr();
		print $q->end_table();

		$q->start_message_table("warning");
		print $q->start_p({ class => "message-description-text" });
		print gettext("This action will take the device group offline");
		print $q->end_p();
		$q->end_message_table();
		
		$q->buttons_table(gettext(" Offline Device Group "),
                	"document.inputs.submit()",
			gettext("  Cancel  "),
			"self.location.href = \"$VIEW\"" );
		print "</form>";
	} else {
		# Generate command
		my $cmd = "$SCSWITCH -F -D $devicegroup"; 
	
		# Executing the command
		$self->execute_cmd($q, $cmd, "Offline", "",
			[gettext("Global Devices"), $VIEW,
			    gettext("Offline Device Group"),
		     "$CONFIG\?step=1&devicegroup=$devicegroup&func=Offline"]);
	}
}

#sub config_remove
#{
#	my ($self, $q, $item_type, $item_name) = @_;
#
#	my ($cmd);
#	
#	$cmd = "$SCCONF -r -D name=$item_name";
#	
#	print $q->center();
#	print $q->font({ color => "black" });
#	print sprintf(gettext("The command to remove the %s(%s) is:"), $item_type, $item_name); 
#	print $q->br(), $q->br(), $q->start_i();
#	print $cmd;
#	print $q->br(),$q->br(), $q->end_i();
#	print $q->end_font();
#	print "<FORM>";
#	print "<INPUT TYPE=button VALUE=".gettext("Back")." onClick=\"history.back()\">"; 
#	print $q->indent(3);
#	# replace space with + for passing argument
#	$cmd =~ tr/ /+/;
#	print "<INPUT TYPE=button VALUE=\"".gettext("Continue")."\" onClick=\"self.location.href='$CONFIG\?type=$item_type&name=$item_name&action=Run&cmd=$cmd'\">"; 
#	print "</FORM>";
#
#	print $q->end_font();
#	print $q->end_center();
#}

# Get a list of nodes from a string with node name separated by ,
sub get_nodelist_fromstr
{
	my ($self, $nodeliststr) = @_;
	my (@node_list, @parts); 
	my $idx = 0;
	@parts = split /(\,)/, $nodeliststr;
	for (my $i = 0; $i < $#parts + 1; $i = $i + 2) {
		$node_list[$idx++] = $parts[$i];
	}
	return (@node_list);
}

sub execute_cmd($$$$$)
{
	my ($self, $q, $cmd, $action, $newprimary, $title) = @_;

	my $output = `$PFEXEC $cmd 2>&1`;
	my $ret = $?;
	if ($ret == 0) {
		pop @$title; # Don't use last link
		$q->print_title($title);
                print "<form>";
                $q->start_message_table("info");
                print $q->start_p({ class => "message-header-text" });
		if ($action eq "Online") {
			print sprintf(gettext("Primary Successfully Set to %s"), $newprimary);
		} else {
			print sprintf(gettext("Device Group Successfully Taken %s"),
			    gettext($action));
		}
                print $q->end_p();
                print $q->start_p({ class => "message-description-text" });
                print gettext("The following command completed without error:");
                print $q->end_p();
                print $q->start_p({ class => "message-command-text" });
                print $cmd;
                print $q->end_p();
                $q->end_message_table();
                $q->buttons_table(gettext(" View Device Group Information "),
                                "self.location.href= \"$VIEW\"" );
                print "</form>";
	} else {
		push @$title, gettext("Error"); # Add error to title
		$q->print_title($title);
                print "<form>";
                $q->start_message_table("error");
                print $q->start_p({ class => "message-header-text" });
		if ($action eq "Online") {
			print sprintf(gettext("Primary Not Set to %s"), $newprimary);
		} else {
			print sprintf(gettext("Device Group Not Taken %s"),
				gettext($action));
		}
                print $q->end_p();
                print $q->start_p({ class => "message-description-text" });
                print gettext("The command:");
                print $q->end_p();
                print $q->start_p({ class => "message-command-text" });
                print $cmd;
                print $q->end_p();
                print $q->start_p({ class => "message-description-text" });
                print gettext("Failed with the following error:");
                print $q->end_p();
                print $q->start_p({ class => "message-command-text" });
                print $output;
                print $q->end_p();
                $q->end_message_table();

		$q->buttons_table(gettext("  < Back  "), "history.back()",
                        gettext("  Cancel  "),
                        "self.location.href = \"$VIEW\"" );
                print "</form>";
        }
}

# Get useful commands output into memory 
sub get_commandoutput
{
	my ($self) = @_;
	my $updated;
	($updated, @SCCONFPOUT) = $runcommand->scconfp_cache();

	if ($updated == TRUE) {
		@DG_LIST = $self->_get_devicegrouplist();
	}	
}

##############################################################################
#
# Updated Cluster Manager Look and Feel functions.
#
# See http://spgweb.eng/hci/behret/sc3x-cm/ for more information.
#
##############################################################################

#
# Print the global devices table
#

sub global_devices_table($$) {
	my ($self, $q) = @_;

	# Set the headers for the device group status table. 
	my @deviceheaders = (gettext("Name"), "33%",
			 gettext("Status"), "20%",
			 gettext("Current Primary"), "47%");

	# Determine the number of columns from the headers
	my $numcols = ($#deviceheaders + 1) / 2;

	# Start the property table
	$q->start_prop_table(gettext("Global Devices"), \@deviceheaders);

	# Get the list of device groups. Each element of the returned
	# array is hash table, containing the fields of: name, status,
	# primary, and nodelist.

	# Load the device data
	my @devlist = $self->get_devicegrouplist();

	# If we don't have any device groups, print a message
	if (!defined @devlist || $#devlist == -1) {
		# Start a table row and cell with the colspan of the table
		$q->start_prop_tr();
		$q->start_prop_td({ COLSPAN => 3 });

		# Print the message in table-label-text style
		$q->start_table_text('table-label-text');
		print gettext("No global devices have been configured on this cluster");
		$q->end_table_text();

		# End the cell and row
		$q->end_prop_td();
		$q->end_prop_tr();
	}

	# Cycle through the list of device groups and print the status
	for (my $i=0; $i<=$#devlist; $i++) {

		# Get the node status
		my ($name, $status, $primary, $nodelist);

		$name = $devlist[$i]{'name'};
		$status = $devlist[$i]{'status'};
		$primary = $devlist[$i]{'primary'};
		$nodelist = $devlist[$i]{'nodelist'};

		# Start a table row for the devicegroup
		$q->start_prop_tr();

		# Print a table cell for the nodename
		$q->start_prop_td();

		# This function will print an icon and a link. It takes the
		# label text, the alt tag text, the link to point to, and the
		# icon to display before the text.
		$q->table_link_text($name, 
				    gettext("Global Device"),
				    "$VIEW?devicegroup=$devlist[$i]{'name'}",
			    "$DEVICEIMAGE"); 

		$q->end_prop_td();

		# Print a table cell for the status
		$q->start_prop_td();

		# This function will print the correctly-colored & styled
		# text for the given status.
		$q->table_status_text($status);

		# End the table cell and table row
		$q->end_prop_td();

		# Start the current primaries table cell
		$q->start_prop_td();

		# Print the primary
		$q->start_table_text('table-normal-text');
		print $primary;
		$q->end_table_text();

		# End the cell and row
		$q->end_prop_td();
		$q->end_prop_tr();

		# If we're not on our last row, print the line row divider
		if ($i < $#devlist) {
		    $q->line_row($numcols);
		}
	}
	$q->end_prop_table();
}

# Print the specified devicegroup's properties 
sub devicegroup_table($$$)
{
	my ($self, $q, $devicegroup) = @_;

	# Set the headers for the device group property table 
        my @headers = (gettext("Property"), "50%",
                         gettext("Value"), "50%");

	# Determine the number of columns from the headers
        my $numcols = ($#headers + 1) / 2;

	# Start the property table
        $q->start_prop_table("Global Device Properties - $devicegroup",
			\@headers, $DEVICEIMAGE);

	# Load the device data
        my @dg_list = $self->get_devicegrouplist();

	my $idx = $self->find_dg_by_name(\@dg_list, $devicegroup);
	if ($idx < 0) {
		# If we can't find the devicegroup name, print error 
		# Start a table row and cell with the colspan of the table
		$q->start_prop_tr();
		$q->start_prop_td({ COLSPAN => 3 });

		# Print the message in table-label-text style
		$q->start_table_text('table-label-text');
		print sprintf(gettext("Global Device %s is not configured on this cluster"), $devicegroup);
		$q->end_table_text();

		# End the cell and row
		$q->end_prop_td();
		$q->end_prop_tr();
	        $q->end_prop_table();
		return;
        }

	$q->start_prop_tr();
	$q->start_prop_td();
	$q->start_table_text('table-label-text');
	print gettext("Current Status:");
	$q->end_table_text();
        $q->end_prop_td();
        $q->start_prop_td();
	$q->table_status_text($dg_list[$idx]{'status'});
	$q->end_prop_td();
        $q->end_prop_tr();
        $q->line_row($numcols);
	
	$q->start_prop_tr();
	$q->start_prop_td();
	$q->start_table_text('table-label-text');
	print gettext("Device Group Name:");
	$q->end_table_text();
        $q->end_prop_td();
        $q->start_prop_td();
	$q->start_table_text('table-normal-text');
	print $dg_list[$idx]{'name'};
	$q->end_table_text();
	$q->end_prop_td();
        $q->end_prop_tr();
        $q->line_row($numcols);
	
	$q->start_prop_tr();
	$q->start_prop_td();
	$q->start_table_text('table-label-text');
	print gettext("Device Group Type:");
	$q->end_table_text();
        $q->end_prop_td();
        $q->start_prop_td();
	$q->start_table_text('table-normal-text');
	print $dg_list[$idx]{'type'};
	$q->end_table_text();
	$q->end_prop_td();
        $q->end_prop_tr();
        $q->line_row($numcols);
	
	$q->start_prop_tr();
	$q->start_prop_td();
	$q->start_table_text('table-label-text');
	print gettext("Primary:");
	$q->end_table_text();
        $q->end_prop_td();
        $q->start_prop_td();
	$q->start_table_text('table-normal-text');
	print $dg_list[$idx]{'primary'};
	$q->end_table_text();
	$q->end_prop_td();
        $q->end_prop_tr();
        $q->line_row($numcols);
	
	$q->start_prop_tr();
	$q->start_prop_td();
	$q->start_table_text('table-label-text');
	print gettext("Nodelist:");
	$q->end_table_text();
        $q->end_prop_td();
        $q->start_prop_td();
	$q->start_table_text('table-normal-text');
	print $dg_list[$idx]{'nodelist'};
	$q->end_table_text();
	$q->end_prop_td();
        $q->end_prop_tr();
        $q->line_row($numcols);
	
	$q->start_prop_tr();
	$q->start_prop_td();
	$q->start_table_text('table-label-text');
	print gettext("Failback:");
	$q->end_table_text();
        $q->end_prop_td();
        $q->start_prop_td();
	$q->start_table_text('table-normal-text');
	print $dg_list[$idx]{'failback'};
	$q->end_table_text();
	$q->end_prop_td();
        $q->end_prop_tr();

	# Print mounted file system if there is any
	my @mounted = $self->get_mounted_directories($devicegroup);
	for (my $i = 0; $i <= $#mounted; $i++) {
		$q->line_row($numcols);
		$q->start_prop_tr();
		$q->start_prop_td();
		$q->start_table_text('table-label-text');
		print gettext("Logical Disk:");
		$q->end_table_text();
		$q->end_prop_td();
		$q->start_prop_td();
		$q->start_table_text('table-normal-text');
		print $mounted[$i]{'logicaldisk'};
		$q->end_table_text();
		$q->end_prop_td();
		$q->end_prop_tr();
		$q->line_row($numcols);
	
		$q->start_prop_tr();
		$q->start_prop_td();
		$q->start_table_text('table-label-text');
		print gettext("Mounted Filesystem:");
		$q->end_table_text();
		$q->end_prop_td();
		$q->start_prop_td();
		$q->start_table_text('table-normal-text');
		print $mounted[$i]{'mounted'};
		$q->end_table_text();
		$q->end_prop_td();
		$q->end_prop_tr();
	}
	$q->end_prop_table();
}
	
# Return true
1;
